@@ -73,14 +73,16 @@ mod tests {
     use super::*;
 
     use cool_asserts::assert_matches;
+    use nom::error::ErrorKind as NomErrorKind;
     use nom::Err as NomErr;
 
     use ross_config::extractor::{EventCodeExtractor, PacketExtractor};
     use ross_config::filter::ValueEqualToConstFilter;
+    use ross_config::matcher::Matcher;
     use ross_config::producer::PacketProducer;
     use ross_config::Value;
 
-    use crate::error::{ErrorKind, Expectation};
+    use crate::error::ErrorKind;
 
     #[test]
     fn provided_extractor_test() {
@@ -116,8 +118,8 @@ mod tests {
             ),
             Err(NomErr::Error(ParserError::Base {
                 location: _,
-                kind: ErrorKind::Expected(Expectation::Symbol('}')),
-                child: None,
+                kind: ErrorKind::Nom(NomErrorKind::Alt),
+                child: Some(_),
             }))
         );
     }
@@ -150,7 +152,7 @@ mod tests {
             } if match {
                 EventCodeExtractor();
                 ValueEqualToConstFilter(0x0123~u16);
-            }",
+            }input",
         )
         .unwrap();
 
@@ -165,13 +167,13 @@ mod tests {
         );
         assert_matches!(
             creator.matcher,
-            Some(matcher) => {
+            Some(Matcher::Single{extractor, filter}) => {
                 assert_eq!(
-                    format!("{:?}", matcher.extractor),
+                    format!("{:?}", extractor),
                     format!("{:?}", EventCodeExtractor::new())
                 );
                 assert_eq!(
-                    format!("{:?}", matcher.filter),
+                    format!("{:?}", filter),
                     format!("{:?}", ValueEqualToConstFilter::new(Value::U16(0x0123)))
                 );
             },
